# -*- coding: utf-8 -*-
from typing import List

from qqbot.model.member import User, Member


class Message:
    def __init__(self, data=None):
        self.id: str = ""
        self.channel_id: str = ""
        self.guild_id: str = ""
        self.content: str = ""
        self.timestamp: str = ""
        self.edited_timestamp: str = ""
        self.author: User = User()
        self.attachments: List[MessageAttachment] = [MessageAttachment()]
        self.embeds: List[MessageEmbed] = [MessageEmbed()]
        self.mentions: List[User] = [User()]
        self.member: Member = Member()
        self.ark: MessageArk = MessageArk()
        if data:
            self.__dict__ = data


class TypesEnum:
    around = "around"
    before = "before"
    after = "after"
    latest = ""


class MessagesPager:
    def __init__(self, type: TypesEnum, id: str, limit: str):
        self.type = type
        self.id = id
        self.limit = limit


class MessageAttachment:
    def __init__(self, data=None):
        self.url: str = ""
        if data:
            self.__dict__ = data


class MessageEmbed:
    def __init__(self, data=None):
        # 标题
        self.title: str = ""
        # 消息弹窗内容
        self.prompt: str = ""
        # 缩略图
        self.thumbnail: MessageEmbedThumbnail = MessageEmbedThumbnail()
        # 消息创建时间
        self.fields: List[MessageEmbedField] = [MessageEmbedField()]
        if data:
            self.__dict__ = data


class MessageEmbedThumbnail:
    def __init__(self, data=None):
        # 图片地址
        self.url: str = ""
        if data is not None:
            self.__dict__ = data


class MessageEmbedField:
    def __init__(self, data=None):
        self.key: str = ""
        self.value: str = ""
        if data:
            self.__dict__ = data


class MessageArk:
    def __init__(self, data=None):
        self.template_id = 0
        self.kv: List[MessageArkKv] = [MessageArkKv()]
        if data:
            self.__dict__ = data


class MessageArkKv:
    def __init__(self, data=None):
        self.key: str = ""
        self.value: str = ""
        self.obj: List[MessageArkObj] = [MessageArkObj()]
        if data:
            self.__dict__ = data


class MessageArkObj:
    def __init__(self, data=None):
        self.obj_kv: List[MessageArkObjKv] = [MessageArkObjKv()]
        if data:
            self.__dict__ = data


class MessageArkObjKv:
    def __init__(self, data=None):
        self.key: str = ""
        self.value: str = ""
        if data:
            self.__dict__ = data


class MessageSendRequest:
    def __init__(
        self,
        content: str,
        msg_id: str = None,
        embed: MessageEmbed = None,
        ark: MessageArk = None,
        image: str = "",
    ):
        """

        :param content:消息内容，文本内容，支持内嵌格式
        :param msg_id:要回复的消息id(Message.id), 在 AT_CREATE_MESSAGE 事件中获取。带了 msg_id 视为被动回复消息，否则视为主动推送消息
        :param embed:embed 消息，一种特殊的 ark
        :param ark:ark 消息
        :param image:图片url地址
        """

        self.content = content
        self.embed = embed
        self.ark = ark
        self.image = image
        self.msg_id = msg_id


class DirectMessageGuild:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.channel_id: str = ""
        self.creat_time: str = ""
        if data is not None:
            self.__dict__ = data


class CreateDirectMessageRequest:
    def __init__(self, source_guild_id: str, user_id: str):
        """
        :param source_guild_id: 创建的私信频道ID
        :param user_id: 私信接收人用户ID
        """
        self.recipient_id = user_id
        self.source_guild_id = source_guild_id



# -*- coding: utf-8 -*-

import asyncio

from qqbot.core.network.ws_async.ws_async_client import Client
from qqbot.core.network.ws_async.ws_async_pool import SessionPool
from qqbot.core.network.ws.dto.enum_intents import Intents
from qqbot.core.network.ws.ws_session import Session, ShardConfig
from qqbot.core.util import logging
from qqbot.model.token import Token

logger = logging.getLogger()


def _loop_exception_handler(loop, context):
    # first, handle with default handler
    loop.default_exception_handler(context)

    exception = context.get("exception")
    if isinstance(exception, ZeroDivisionError):
        print(context)
        loop.stop()


async def _on_connected(ws_client):
    if ws_client.ws_conn is None:
        raise Exception("websocket connection failed ")
    if ws_client.session.session_id != "":
        await ws_client.reconnect()
    else:
        await ws_client.identify()


def _check_session_limit(websocket_ap):
    return websocket_ap["shards"] > websocket_ap["session_start_limit"]["remaining"]


def _cal_interval(max_concurrency):
    """
    :param max_concurrency:每5s可以创建的session数
    :return: 链接间隔时间
    """
    return round(5 / max_concurrency)


class SessionManager:
    session_pool: SessionPool

    def start(self, websocket_ap, token=Token, intent=Intents):
        logger.info(
            "session manager start with: %s" % websocket_ap
            + ", token:%s" % token
            + ", intent:%s" % intent
        )
        # 每个机器人创建的连接数不能超过remaining剩余连接数
        if _check_session_limit(websocket_ap):
            raise Exception("session limit exceeded")
        # 根据session限制建立链接
        session_interval = _cal_interval(
            websocket_ap["session_start_limit"]["max_concurrency"]
        )
        shards_count = websocket_ap["shards"]
        logger.info(
            "session_interval: %s, shards: %s" % (session_interval, shards_count)
        )
        # 根据限制建立分片的并发链接数
        self.init_session_pool(
            intent, shards_count, token, websocket_ap, session_interval
        )

    def init_session_pool(
        self, intent, shards_count, token, websocket_ap, session_interval
    ):

        # 实例一个session_pool
        self.session_pool = SessionPool(
            max_async=websocket_ap["session_start_limit"]["max_concurrency"],
            session_manager=self,
            loop=asyncio.get_event_loop(),
        )
        for i in range(shards_count):
            session = Session(
                session_id="",
                url=websocket_ap["url"],
                intent=intent,
                last_seq=0,
                token=token,
                shards=ShardConfig(i, shards_count),
            )
            self.session_pool.add(session)
        self.start_session(session_interval)

    def start_session(self, session_interval=5):
        pool = self.session_pool
        loop = pool.loop
        loop.set_exception_handler(_loop_exception_handler)
        try:
            loop.run_until_complete(pool.run(session_interval))
            loop.run_forever()
        except KeyboardInterrupt:
            logger.info("ws pool is stopped by key board interrupt")
            # cancel all tasks lingering

    async def new_connect(self, session, time_interval):
        """
        newConnect 启动一个新的连接，如果连接在监听过程中报错了，或者被远端关闭了链接，需要识别关闭的原因，能否继续 resume
        如果能够 resume，则往 sessionChan 中放入带有 sessionID 的 session
        如果不能，则清理掉 sessionID，将 session 放入 sessionChan 中
        session 的启动，交给 start 中的 for 循环执行，session 不自己递归进行重连，避免递归深度过深

        param session: session对象
        """
        await asyncio.sleep(time_interval)
        logger.info("_new_connect:%s" % session)

        client = Client(session, self, _on_connected)
        try:
            await client.connect()
        except (Exception, KeyboardInterrupt, SystemExit) as e:
            await client.on_error(e)



# -*- coding: utf-8 -*-
from .audio import AudioControl, STATUS
from .channel import ChannelType, ChannelSubType, PatchChannelRequest
from .channel import CreateChannelRequest
from .channel_permissions import UpdatePermission
from .guild_member import QueryParams
from .guild_role import RoleUpdateInfo
from .message import CreateDirectMessageRequest, MessageSendRequest, Message
from .mute import MuteOption
from .token import Token
from .user import ReqOption



# -*- coding: utf-8 -*-


class MuteOption:
    def __init__(self, mute_end_timestamp: str = None, mute_seconds: str = None):
        """

        :param mute_end_timestamp: 禁言到期时间戳，绝对时间戳，单位：秒（与 seconds 字段同时赋值的话，以该字段为准）
        :param mute_seconds: 禁言多少秒（两个字段二选一，默认以 timeTo 为准）
        """
        self.mute_end_timestamp = mute_end_timestamp
        self.mute_seconds = mute_seconds



# -*- coding: utf-8 -*-


class Announce:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.channel_id: str = ""
        self.message_id: str = ""
        if data:
            self.__dict__ = data


class CreateAnnounceRequest:
    def __init__(self, channel_id: str, message_id: str):
        self.channel_id = channel_id
        self.message_id = message_id


class CreateChannelAnnounceRequest:
    def __init__(self, message_id: str):
        self.message_id = message_id



# -*- coding: utf-8 -*-

import json


class JsonUtil:
    @staticmethod
    def json_str2obj(json_str, obj):
        """
        转换的对象默认值会带上，建议还是用json.loads(object_hook)的方式
        """
        py_data = json.loads(json_str)
        _dic2class(py_data, obj)
        return obj

    @staticmethod
    def obj2json_str(obj):
        return json.dumps(obj, default=_convert_to_builtin_type)

    @staticmethod
    def obj2json_serialize(obj):
        return json.loads(JsonUtil.obj2json_str(obj))


def _dic2class(py_data, obj):
    for name in [name for name in dir(obj) if not name.startswith("_")]:
        if name not in py_data:
            setattr(obj, name, None)
        else:
            value = getattr(obj, name)
            setattr(obj, name, _set_value(value, py_data[name]))


def _set_value(value, py_data):
    if str(type(value)).__contains__("."):
        # value 为自定义类
        _dic2class(py_data, value)
    elif str(type(value)) == "<class 'list'>":
        # value为列表
        if value.__len__() == 0:
            # value列表中没有元素，无法确认类型
            value = py_data
        else:
            # value列表中有元素，以第一个元素类型为准
            child_value_type = type(value[0])
            value.clear()
            for child_py_data in py_data:
                child_value = child_value_type()
                child_value = _set_value(child_value, child_py_data)
                value.append(child_value)
    else:
        value = py_data
    return value


def _convert_to_builtin_type(obj):
    d = {}
    d.update(obj.__dict__)
    return d



# -*- coding: utf-8 -*-

# 异步api
import json
from typing import List

import aiohttp

from qqbot import WebsocketAPI
from qqbot.core.network.async_http import AsyncHttp
from qqbot.core.network.url import get_url, APIConstant
from qqbot.core.network.ws.ws_intents_handler import Handler, register_handlers
from qqbot.core.network.ws_async.ws_async_manager import SessionManager
from qqbot.core.util.json_util import JsonUtil
from qqbot.model.announce import (
    CreateAnnounceRequest,
    Announce,
    CreateChannelAnnounceRequest,
)
from qqbot.model.audio import AudioControl
from qqbot.model.channel import (
    Channel,
    ChannelResponse,
    CreateChannelRequest,
    PatchChannelRequest,
)
from qqbot.model.channel_permissions import (
    ChannelPermissions,
    UpdatePermission,
)
from qqbot.model.guild import Guild
from qqbot.model.guild_member import QueryParams
from qqbot.model.guild_role import (
    GuildRoles,
    RoleUpdateResult,
    RoleUpdateRequest,
    RoleUpdateFilter,
    RoleUpdateInfo,
)
from qqbot.model.member import User, Member
from qqbot.model.message import (
    MessageSendRequest,
    Message,
    CreateDirectMessageRequest,
    DirectMessageGuild,
    MessagesPager,
)
from qqbot.model.mute import MuteOption
from qqbot.model.token import Token
from qqbot.model.user import ReqOption


def async_listen_events(t_token: Token, is_sandbox: bool, *handlers: Handler):
    """
    异步注册并监听频道相关事件

    :param t_token: Token对象
    :param handlers: 包含事件类型和事件回调的Handler对象，支持多个对象
    :param is_sandbox:是否沙盒环境，默认为False
    """
    # 通过api获取websocket链接
    ws_api = WebsocketAPI(t_token, is_sandbox)
    ws_ap = ws_api.ws()
    # 新建和注册监听事件
    t_intent = register_handlers(handlers)
    # 实例一个session_manager
    manager = SessionManager()
    manager.start(ws_ap, t_token.bot_token(), t_intent)


class AsyncAPIBase:
    timeout = 3
    client_session = aiohttp.ClientSession()

    def __init__(self, token: Token, is_sandbox: bool):
        """
        API初始化信息

        :param token: Token对象
        :param is_sandbox: 是否沙盒环境
        """
        self.is_sandbox = is_sandbox
        self.token = token
        self.http_async = AsyncHttp(
            self.client_session, self.timeout, token.get_string(), token.get_type()
        )

    def with_timeout(self, timeout):
        self.timeout = timeout
        return self


class AsyncGuildAPI(AsyncAPIBase):
    """
    频道相关接口
    """

    async def get_guild(self, guild_id: str) -> Guild:
        """
        获取频道信息

        :param guild_id: 频道ID（一般从事件中获取相关的ID信息）
        :return: 频道Guild对象
        """
        url = get_url(APIConstant.guildURI, self.is_sandbox).format(guild_id=guild_id)
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=Guild)


class AsyncGuildRoleAPI(AsyncAPIBase):
    """
    频道身份组相关接口
    """

    async def get_guild_roles(self, guild_id: str) -> GuildRoles:
        """
        获取频道身份组列表

        :param guild_id:频道ID
        :return:GuildRoles对象
        """
        url = get_url(APIConstant.rolesURI, self.is_sandbox).format(guild_id=guild_id)
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=GuildRoles)

    async def create_guild_role(
        self, guild_id: str, role_info: RoleUpdateInfo
    ) -> RoleUpdateResult:
        """
        创建频道身份组

        :param guild_id:频道ID
        :param role_info:RoleUpdateInfo对象，需要自己创建的身份数据
        :return:RoleUpdateResult对象
        """
        url = get_url(APIConstant.rolesURI, self.is_sandbox).format(guild_id=guild_id)
        params = RoleUpdateRequest()
        params.filter = RoleUpdateFilter(1, 1, 1)
        params.guild_id = guild_id
        params.info = role_info
        serialize = JsonUtil.obj2json_serialize(params)
        response = await self.http_async.post(url, request=serialize)
        return json.loads(response, object_hook=RoleUpdateResult)

    async def update_guild_role(
        self, guild_id: str, role_id: str, role_info: RoleUpdateInfo
    ) -> RoleUpdateResult:
        """
        修改频道身份组

        :param guild_id:频道ID
        :param role_id:身份组ID
        :param role_info:更新后的RoleUpdateInfo对象
        :return:RoleUpdateResult对象
        """
        url = get_url(APIConstant.roleURI, self.is_sandbox).format(
            guild_id=guild_id, role_id=role_id
        )
        params = RoleUpdateRequest()
        params.filter = RoleUpdateFilter(1, 1, 1)
        params.guild_id = guild_id
        params.info = role_info
        serialize = JsonUtil.obj2json_serialize(params)
        response = await self.http_async.patch(url, request=serialize)
        return json.loads(response, object_hook=RoleUpdateResult)

    async def delete_guild_role(self, guild_id: str, role_id: str) -> bool:
        """
        删除频道身份组

        :param guild_id: 频道ID
        :param role_id: 身份组ID
        :return: 是否删除成功
        """
        url = get_url(APIConstant.roleURI, self.is_sandbox).format(
            guild_id=guild_id, role_id=role_id
        )
        response = await self.http_async.delete(url)
        return response == ""

    async def create_guild_role_member(
        self,
        guild_id: str,
        role_id: str,
        user_id: str,
        role_req: Channel = None,
    ) -> bool:
        """
        增加频道身份组成员
        需要使用的 token 对应的用户具备删除身份组成员权限。如果是机器人，要求被添加为管理员。
        如果要删除的身份组ID是5-子频道管理员，需要增加channel对象来指定具体是哪个子频道

        :param guild_id:频道ID
        :param role_id:身份组ID
        :param user_id:用户ID
        :param role_req:RoleMemberRequest数据对象
        :return:是否添加成功
        """
        url = get_url(APIConstant.memberRoleURI, self.is_sandbox).format(
            guild_id=guild_id, role_id=role_id, user_id=user_id
        )
        response = await self.http_async.put(
            url, request=JsonUtil.obj2json_serialize(role_req)
        )
        return response == ""

    async def delete_guild_role_member(
        self,
        guild_id: str,
        role_id: str,
        user_id: str,
        role_req: Channel = None,
    ) -> bool:
        """
        删除频道身份组成员
        需要使用的 token 对应的用户具备删除身份组成员权限。如果是机器人，要求被添加为管理员。
        如果要删除的身份组ID是5-子频道管理员，需要增加channel对象来指定具体是哪个子频道

        :param guild_id:频道ID
        :param role_id:身份组ID
        :param user_id:用户ID
        :param role_req:RoleMemberRequest数据对象
        :return:是否删除成功
        """
        url = get_url(APIConstant.memberRoleURI, self.is_sandbox).format(
            guild_id=guild_id, role_id=role_id, user_id=user_id
        )
        response = await self.http_async.delete(
            url, request=JsonUtil.obj2json_serialize(role_req)
        )
        return response == ""


class AsyncGuildMemberAPI(AsyncAPIBase):
    """
    成员相关接口，添加成员到用户组等
    """

    async def get_guild_member(self, guild_id: str, user_id: str) -> Member:
        """
        获取频道指定成员

        :param guild_id:频道ID
        :param user_id:用户ID（一般从事件消息中获取）
        :return:
        """
        url = get_url(APIConstant.guildMemberURI, self.is_sandbox).format(
            guild_id=guild_id, user_id=user_id
        )
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=Member)

    async def get_guild_members(
        self, guild_id: str, guild_member_pager: QueryParams
    ) -> List[Member]:
        """
        获取成员列表，需要申请接口权限

        :param guild_id: 频道ID
        :param guild_member_pager: GuildMembersPager分页数据对象
        :return: Member列表
        """
        url = get_url(APIConstant.guildMembersURI, self.is_sandbox).format(
            guild_id=guild_id
        )
        response = await self.http_async.get(url, params=guild_member_pager.__dict__)
        return json.loads(response, object_hook=Member)


class AsyncChannelAPI(AsyncAPIBase):
    """子频道相关接口"""

    async def get_channel(self, channel_id) -> Channel:
        """
        获取子频道信息

        :param channel_id:子频道ID
        :return:子频道对象Channel
        """
        url = get_url(APIConstant.channelURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=Channel)

    async def get_channels(self, guild_id: str) -> List[Channel]:
        """
        获取频道下的子频道列表

        :param guild_id: 频道ID
        :return: Channel列表
        """
        url = get_url(APIConstant.channelsURI, self.is_sandbox).format(
            guild_id=guild_id
        )
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=Channel)

    async def create_channel(
        self, guild_id: str, request: CreateChannelRequest
    ) -> ChannelResponse:
        """
        创建子频道

        :param guild_id: 频道ID
        :param request: 创建子频道的请求对象CreateChannelRequest
        :return ChannelResponse 对象
        """
        url = get_url(APIConstant.channelsURI, self.is_sandbox).format(
            guild_id=guild_id
        )
        request_json = JsonUtil.obj2json_serialize(request)
        response = await self.http_async.post(url, request_json)
        return json.loads(response, object_hook=ChannelResponse)

    async def update_channel(
        self, channel_id: str, request: PatchChannelRequest
    ) -> ChannelResponse:
        """
        修改子频道

        :param channel_id: 频道ID
        :param request: PatchChannelRequest
        :return ChannelResponse 对象
        """
        url = get_url(APIConstant.channelURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        request_json = JsonUtil.obj2json_serialize(request)
        response = await self.http_async.patch(url, request_json)
        return json.loads(response, object_hook=ChannelResponse)

    async def delete_channel(self, channel_id: str) -> ChannelResponse:
        """
        删除子频道

        :param channel_id: 频道ID
        :return ChannelResponse 对象
        """
        url = get_url(APIConstant.channelURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        response = await self.http_async.delete(url)
        return json.loads(response, object_hook=ChannelResponse)


class AsyncChannelPermissionsAPI(AsyncAPIBase):
    """子频道权限相关接口"""

    async def get_channel_permissions(
        self, channel_id: str, user_id: str
    ) -> ChannelPermissions:
        """
        获取指定子频道的权限

        :param channel_id:子频道ID
        :param user_id:用户ID
        :return:ChannelPermissions对象
        """
        url = get_url(APIConstant.channelPermissionsURI, self.is_sandbox).format(
            channel_id=channel_id, user_id=user_id
        )
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=ChannelPermissions)

    async def update_channel_permissions(
        self, channel_id, user_id, request: UpdatePermission
    ) -> bool:
        """
        修改指定子频道的权限

        :param channel_id:子频道ID
        :param user_id:用户ID
        :param request:ChannelPermissionsUpdateRequest数据对象（构造可以查看具体的对象注释）
        :return:
        """
        url = get_url(APIConstant.channelPermissionsURI, self.is_sandbox).format(
            channel_id=channel_id, user_id=user_id
        )
        if request.add != "":
            request.add = str(int(request.add, 16))
        if request.remove != "":
            request.remove = str(int(request.remove, 16))
        response = await self.http_async.put(
            url, request=JsonUtil.obj2json_serialize(request)
        )
        return response == ""

    async def get_channel_role_permissions(
        self, channel_id: str, role_id: str
    ) -> ChannelPermissions:
        """
        获取指定子频道的权限

        :param channel_id:子频道ID
        :param role_id:身份组ID
        :return:ChannelPermissions对象
        """
        url = get_url(APIConstant.channelRolePermissionsURI, self.is_sandbox).format(
            channel_id=channel_id, role_id=role_id
        )
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=ChannelPermissions)

    async def update_channel_role_permissions(
        self, channel_id: str, role_id: str, request: UpdatePermission
    ) -> bool:
        """
        修改指定子频道的权限

        :param channel_id:子频道ID
        :param role_id:身份组ID
        :param request:ChannelPermissionsUpdateRequest数据对象（构造可以查看具体的对象注释）
        :return:
        """
        url = get_url(APIConstant.channelRolePermissionsURI, self.is_sandbox).format(
            channel_id=channel_id, role_id=role_id
        )
        if request.add != "":
            request.add = str(int(request.add, 16))
        if request.remove != "":
            request.remove = str(int(request.remove, 16))
        response = await self.http_async.put(
            url, request=JsonUtil.obj2json_serialize(request)
        )
        return response == ""


class AsyncMessageAPI(AsyncAPIBase):
    """消息"""

    async def get_message(self, channel_id: str, message_id: str) -> Message:
        """
        获取指定消息

        :param channel_id: 频道ID
        :param message_id: 消息ID
        :return: Message 对象
        """
        url = get_url(APIConstant.messageURI, self.is_sandbox).format(
            channel_id=channel_id, message_id=message_id
        )
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=Message)

    async def get_messages(
        self, channel_id: str, pager: MessagesPager
    ) -> List[Message]:
        """
        获取指定消息列表

        :param channel_id: 频道ID
        :param pager: MessagesPager对象
        :return: Message 对象
        """
        url = get_url(APIConstant.messagesURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        query = {}
        if pager.limit != "":
            query["limit"] = pager.limit

        if pager.type != "" and pager.id != "":
            query[pager.type] = pager.id

        response = await self.http_async.get(url, params=query)
        return json.loads(response, object_hook=Message)

    async def post_message(
        self, channel_id: str, message_send: MessageSendRequest
    ) -> Message:
        """
        发送消息

        要求操作人在该子频道具有发送消息的权限。
        发送成功之后，会触发一个创建消息的事件。
        被动回复消息有效期为 5 分钟
        主动推送消息每日每个子频道限 2 条
        发送消息接口要求机器人接口需要链接到websocket gateway 上保持在线状态

        :param channel_id: 子频道ID
        :param message_send: MessageSendRequest对象
        :return: Message对象
        """

        url = get_url(APIConstant.messagesURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        request_json = JsonUtil.obj2json_serialize(message_send)
        response = await self.http_async.post(url, request_json)
        return json.loads(response, object_hook=Message)

    async def recall_message(self, channel_id: str, message_id: str):
        """
        撤回消息

        管理员可以撤回普通成员的消息
        频道主可以撤回所有人的消息

        :param channel_id: 子频道ID
        :param message_id: 消息ID
        """
        url = get_url(APIConstant.messageURI, self.is_sandbox).format(
            channel_id=channel_id, message_id=message_id
        )
        response = await self.http_async.delete(url)
        return response.status_code == ""

    async def create_direct_message(
        self, create_direct_message: CreateDirectMessageRequest
    ) -> DirectMessageGuild:
        """
        创建私信频道

        :param create_direct_message: 构造request数据
        :return: 私信频道对象
        """
        url = get_url(APIConstant.userMeDMURI, self.is_sandbox)
        request_json = JsonUtil.obj2json_serialize(create_direct_message)
        response = await self.http_async.post(url, request_json)
        return json.loads(response, object_hook=DirectMessageGuild)

    async def post_direct_message(
        self, guild_id: str, message_send: MessageSendRequest
    ) -> Message:
        """
        发送私信

        :param guild_id: 创建的私信频道id
        :param message_send: 发送消息的数据请求对象 MessageSendRequest
        :return Message对象
        """
        url = get_url(APIConstant.dmsURI, self.is_sandbox).format(guild_id=guild_id)
        request_json = JsonUtil.obj2json_serialize(message_send)
        response = await self.http_async.post(url, request_json)
        return json.loads(response, object_hook=Message)


class AsyncAudioAPI(AsyncAPIBase):
    """音频接口"""

    async def post_audio(self, channel_id: str, audio_control: AudioControl) -> bool:
        """
        音频控制

        :param channel_id:频道ID
        :param audio_control:AudioControl对象
        :return:是否成功
        """
        url = get_url(APIConstant.audioControlURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        request_json = JsonUtil.obj2json_serialize(audio_control)
        response = await self.http_async.post(url, request=request_json)
        return response == ""


class AsyncUserAPI(AsyncAPIBase):
    """用户相关接口"""

    async def me(self) -> User:
        """
        :return:使用当前用户信息填充的 User 对象
        """
        url = get_url(APIConstant.userMeURI, self.is_sandbox)
        response = await self.http_async.get(url)
        return json.loads(response, object_hook=User)

    async def me_guilds(self, option: ReqOption = None) -> List[Guild]:
        """
        当前用户所加入的 Guild 对象列表

        :param option: ReqOption对象
        :return:Guild对象列表
        """
        url = get_url(APIConstant.userMeGuildsURI, self.is_sandbox)
        request_json = JsonUtil.obj2json_serialize(option)
        response = await self.http_async.get(url, request=request_json)
        return json.loads(response, object_hook=Guild)


class AsyncWebsocketAPI(AsyncAPIBase):
    """WebsocketAPI"""

    async def ws(self):
        url = get_url(APIConstant.gatewayBotURI, self.is_sandbox)
        response = await self.http_async.get(url)
        websocket_ap = json.loads(response)
        return websocket_ap


class AsyncMuteAPI(AsyncAPIBase):
    """禁言接口"""

    async def mute_all(self, guild_id: str, options: MuteOption):
        """
        禁言全员

        :param guild_id: 频道ID
        :param options: MuteOptions对象
        """
        url = get_url(APIConstant.guildMuteURI, self.is_sandbox).format(
            guild_id=guild_id
        )
        request_json = JsonUtil.obj2json_serialize(options)
        response = await self.http_async.patch(url, request=request_json)
        return response == ""

    async def mute_member(self, guild_id: str, user_id: str, options: MuteOption):
        """
        禁言指定成员

        :param guild_id: 频道ID
        :param user_id: 用户ID
        :param options: MuteOptions对象
        """
        url = get_url(APIConstant.guildMemberMuteURI, self.is_sandbox).format(
            guild_id=guild_id, user_id=user_id
        )
        request_json = JsonUtil.obj2json_serialize(options)
        response = await self.http_async.patch(url, request=request_json)
        return response == ""


class AsyncAnnouncesAPI(AsyncAPIBase):
    """公告接口"""

    async def create_announce(
        self, guild_id: str, request: CreateAnnounceRequest
    ) -> Announce:
        """
        创建频道全局公告

        :param guild_id: 频道ID
        :param request: CreateAnnounceRequest对象
        """
        url = get_url(APIConstant.guildAnnounceURI, self.is_sandbox).format(
            guild_id=guild_id
        )
        request_json = JsonUtil.obj2json_serialize(request)
        response = await self.http_async.post(url, request_json)
        return json.loads(response, object_hook=Announce)

    async def delete_announce(self, guild_id: str, message_id: str):
        """
        删除频道全局公告
        message_id 有值时，会校验 message_id 合法性，若不校验校验 message_id，请将 message_id 设置为 all

        :param guild_id: 频道ID
        :param message_id: 消息ID
        """
        url = get_url(APIConstant.deleteGuildAnnounceURI, self.is_sandbox).format(
            guild_id=guild_id, message_id=message_id
        )
        response = await self.http_async.delete(url)
        return response == ""

    async def create_channel_announce(
        self, channel_id: str, request: CreateChannelAnnounceRequest
    ) -> Announce:
        """
        设置消息为指定子频道公告

        :param channel_id: 频道ID
        :param request: CreateChannelAnnounceRequest对象
        """
        url = get_url(APIConstant.channelAnnounceURI, self.is_sandbox).format(
            channel_id=channel_id
        )
        request_json = JsonUtil.obj2json_serialize(request)
        response = await self.http_async.post(url, request_json)
        return json.loads(response, object_hook=Announce)

    async def delete_channel_announce(self, channel_id: str, message_id: str):
        """
        删除子频道公告
        message_id 有值时，会校验 message_id 合法性，若不校验校验 message_id，请将 message_id 设置为 all

        :param channel_id: 频道ID
        :param message_id: 消息ID
        """
        url = get_url(APIConstant.deleteChannelAnnounceURI, self.is_sandbox).format(
            channel_id=channel_id, message_id=message_id
        )
        response = await self.http_async.delete(url)
        return response == ""



# -*- coding: utf-8 -*-
import asyncio
import json
import traceback

import aiohttp
from aiohttp import WSMessage, ClientWebSocketResponse

from qqbot.core.exception.error import WebsocketError
from qqbot.core.network.ws.dto.enum_intents import Intents
from qqbot.core.network.ws.dto.enum_opcode import OpCode
from qqbot.core.network.ws.dto.ws_payload import (
    WSPayload,
    WsIdentifyData,
    WSResumeData,
)
from qqbot.core.network.ws_async.ws_async_handler import parse_and_handle
from qqbot.core.util import logging

logger = logging.getLogger()


class Client:
    def __init__(self, session, session_manager, connected_callback=None):
        self.session = session
        self.ws_conn = None
        self.session_manager = session_manager
        self.connected_callback = connected_callback
        self.can_reconnect = False

    async def on_error(self, exception: BaseException):
        logger.error(
            "on_error: websocket connection: %s, exception : %s"
            % (self.ws_conn, exception)
        )
        traceback.print_exc()

    async def on_close(self, ws, close_status_code, close_msg):
        logger.info(
            "on_close: websocket connection %s" % ws
            + ", code: %s" % close_status_code
            + ", msg: %s" % close_msg
        )
        # 这种不能重新链接
        if (
            close_status_code == WebsocketError.CodeConnCloseErr
            or close_status_code == WebsocketError.CodeInvalidSession
            or self.can_reconnect is False
        ):
            self.session.session_id = ""
            self.session.last_seq = 0
        # 断连后启动一个新的链接并透传当前的session，不使用内部重连的方式，避免死循环
        self.session_manager.session_pool.add(self.session)
        asyncio.ensure_future(self.session_manager.session_pool.run())

    async def on_message(self, ws, message):
        logger.info("on_message: %s" % message)
        message_event = json.loads(message)
        if await self._is_system_event(message_event, ws):
            return
        if "t" in message_event.keys() and message_event["t"] == "READY":
            event_seq = message_event["s"]
            if event_seq > 0:
                self.session.last_seq = event_seq
            await self._ready_handler(message_event)
            return
        if "t" in message_event.keys():
            await parse_and_handle(message_event, message)

    async def on_connected(self, ws):
        logger.info("ws client connected ok")
        self.ws_conn = ws
        await self.connected_callback(self)
        # 心跳检查
        asyncio.ensure_future(self._send_heartbeat(interval=30))

    async def connect(self):
        """
        websocket向服务器端发起链接，并定时发送心跳
        """

        logger.info("ws client start connect")
        ws_url = self.session.url
        if ws_url == "":
            raise Exception("session url is none")

        async with aiohttp.ClientSession() as session:
            async with session.ws_connect(self.session.url) as ws_conn:
                while True:
                    msg: WSMessage
                    msg = await ws_conn.receive()
                    await self.dispatch(msg, ws_conn)
                    if ws_conn.closed:
                        logger.info("ws is closed, stop circle receive msg")
                        break

    async def dispatch(self, msg, ws_conn):
        """
        ws事件分发
        """
        if msg.type == aiohttp.WSMsgType.TEXT:
            await self.on_message(ws_conn, msg.data)
        elif msg.type == aiohttp.WSMsgType.ERROR:
            await self.on_error(ws_conn.exception())
        elif (
            msg.type == aiohttp.WSMsgType.CLOSED or msg.type == aiohttp.WSMsgType.CLOSE
        ):
            await self.on_close(ws_conn, ws_conn.close_code, msg.extra)

    async def identify(self):
        """
        websocket鉴权
        """
        if self.session.intent == 0:
            self.session.intent = Intents.INTENT_GUILDS.value
        logger.info("ws:%s start identify" % self.ws_conn)
        identify_event = json.dumps(
            WSPayload(
                WsIdentifyData(
                    token=self.session.token.get_string(),
                    intents=self.session.intent,
                    shard=[
                        self.session.shards.shard_id,
                        self.session.shards.shard_count,
                    ],
                ).__dict__,
                op=OpCode.WS_IDENTITY.value,
            ).__dict__
        )
        await self.send_msg(identify_event)

    async def send_msg(self, event_json):
        """
        websocket发送消息
        :param event_json:
        """
        send_msg = event_json
        logger.info("send_msg: %s" % send_msg)
        if isinstance(self.ws_conn, ClientWebSocketResponse):
            if self.ws_conn.closed:
                logger.error("send_msg: websocket connection has closed")
            else:
                await self.ws_conn.send_str(data=send_msg)

    async def reconnect(self):
        """
        websocket重连
        """
        logger.info("ws:%s is reconnected" % self.ws_conn)
        resume_event = json.dumps(
            WSPayload(
                WSResumeData(
                    token=self.session.token.get_string(),
                    session_id=self.session.session_id,
                    seq=self.session.last_seq,
                ).__dict__,
                op=OpCode.WS_RESUME.value,
            ).__dict__
        )
        await self.send_msg(resume_event)

    async def _ready_handler(self, message_event):
        data = message_event["d"]
        self.version = data["version"]
        self.session.session_id = data["session_id"]
        self.session.shards.shard_id = data["shard"][0]
        self.session.shards.shard_count = data["shard"][1]
        self.user = data["user"]

    async def _is_system_event(self, message_event, ws):
        """
        系统事件
        :param message_event:消息
        :param ws:websocket
        :return:
        """
        event_op = message_event["op"]
        if event_op == OpCode.WS_HELLO.value:
            await self.on_connected(ws)
            return True
        if event_op == OpCode.WS_HEARTBEAT_ACK.value:
            return True
        if event_op == OpCode.WS_RECONNECT.value:
            self.can_reconnect = True
            return True
        if event_op == OpCode.WS_INVALID_SESSION.value:
            self.can_reconnect = False
            return True
        return False

    async def _send_heartbeat(self, interval):
        """
        心跳包
        :param interval: 间隔时间
        """
        logger.info("start send heartbeat")
        while True:
            heartbeat_event = json.dumps(
                WSPayload(
                    op=OpCode.WS_HEARTBEAT.value, d=self.session.last_seq
                ).__dict__
            )
            if self.ws_conn is None:
                logger.error("ws is None")
                return
            else:
                if self.ws_conn.closed:
                    logger.info("ws is closed, stop circle heartbeat")
                    return
                else:
                    await asyncio.sleep(interval)
                    await self.send_msg(heartbeat_event)



# -*- coding: utf-8 -*-
import json

import requests

from qqbot.core.exception.error import (
    AuthenticationFailedError,
    NotFoundError,
    MethodNotAllowedError,
    SequenceNumberError,
    ServerError,
)
from qqbot.core.util import logging

X_TPS_TRACE_ID = "X-Tps-trace-Id"

logger = logging.getLogger()

HttpErrorDict = {
    401: AuthenticationFailedError,
    404: NotFoundError,
    405: MethodNotAllowedError,
    429: SequenceNumberError,
    500: ServerError,
    504: ServerError,
}


class HttpErrorMessage:
    def __init__(self, data=None):
        self.message = ""
        self.code = 0
        if data is not None:
            self.__dict__ = data


class HttpStatus:
    OK = 200
    ACCEPTED = 202
    NO_CONTENT = 204


def _handle_response(api_url, response):
    if response.status_code in (
        HttpStatus.NO_CONTENT,
        HttpStatus.OK,
        HttpStatus.ACCEPTED,
    ):
        return
    else:
        logger.error(
            "http request error with api_url:%s, error: %s, content: %s, trace_id:%s"
            % (
                api_url,
                response.status_code,
                response.content,
                response.headers.get(X_TPS_TRACE_ID),
            )  # trace_id 用于定位接口问题
        )
        error_message_: HttpErrorMessage = json.loads(
            response.content, object_hook=HttpErrorMessage
        )
        error_dict_get = HttpErrorDict.get(response.status_code)
        if error_dict_get is None:
            raise ServerError(error_message_.message)
        raise error_dict_get(msg=error_message_.message)


class Http:
    def __init__(self, time_out, token, type):
        self.timeout = time_out
        self.token = token
        self.scheme = type

    def get(self, api_url, request=None, params=None):
        headers = {
            "Authorization": self.scheme + " " + self.token,
            "User-Agent": "BotPythonSDK/v0.5.4",
        }
        logger.debug("http get headers: %s, api_url: %s" % (headers, api_url))
        response = requests.get(
            url=api_url,
            params=params,
            json=request,
            timeout=self.timeout,
            headers=headers,
        )
        _handle_response(api_url, response)
        return response

    def post(self, api_url, request=None, params=None):
        headers = {
            "Authorization": self.scheme + " " + self.token,
            "User-Agent": "BotPythonSDK/v0.5.4",
        }
        logger.debug(
            "http post headers: %s, api_url: %s, request: %s"
            % (headers, api_url, request)
        )
        response = requests.post(
            url=api_url,
            params=params,
            json=request,
            timeout=self.timeout,
            headers=headers,
        )
        _handle_response(api_url, response)
        return response

    def delete(self, api_url, request=None, params=None):
        headers = {
            "Authorization": self.scheme + " " + self.token,
            "User-Agent": "BotPythonSDK/v0.5.4",
        }
        logger.debug("http delete headers: %s, api_url: %s" % (headers, api_url))
        response = requests.delete(
            url=api_url,
            params=params,
            json=request,
            timeout=self.timeout,
            headers=headers,
        )
        _handle_response(api_url, response)
        return response

    def put(self, api_url, request=None, params=None):
        headers = {
            "Authorization": self.scheme + " " + self.token,
            "User-Agent": "BotPythonSDK/v0.5.4",
        }
        logger.debug(
            "http put headers: %s, api_url: %s, request: %s"
            % (headers, api_url, request)
        )
        response = requests.put(
            url=api_url,
            params=params,
            json=request,
            timeout=self.timeout,
            headers=headers,
        )
        _handle_response(api_url, response)
        return response

    def patch(self, api_url, request=None, params=None):
        headers = {
            "Authorization": self.scheme + " " + self.token,
            "User-Agent": "BotPythonSDK/v0.5.4",
        }
        logger.debug(
            "http patch headers: %s, api_url: %s, request: %s"
            % (headers, api_url, request)
        )
        response = requests.patch(
            url=api_url,
            params=params,
            json=request,
            timeout=self.timeout,
            headers=headers,
        )
        _handle_response(api_url, response)
        return response



# -*- coding: utf-8 -*-
from typing import List


class Role:
    def __init__(self, data=None):
        self.id: str = ""
        self.name: str = ""
        self.color: int = 0
        self.hoist: int = 0
        self.number: int = 0
        self.number_limit: int = 0
        if data is not None:
            self.__dict__ = data


class GuildRoles:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.roles: List[Role] = [Role()]
        self.role_num_limit: str = ""
        if data is not None:
            self.__dict__ = data


class RoleUpdateRequest:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.filter: [RoleUpdateFilter] = None
        self.info: [RoleUpdateInfo] = None
        if data is not None:
            self.__dict__ = data


class RoleUpdateFilter:
    def __init__(self, name: int, color: int, hoist: int):
        self.name = name
        self.color = color
        self.hoist = hoist


class RoleUpdateInfo:
    def __init__(self, name: str, color: int, hoist: int):
        """
        身份组更新的信息

        :param name:名称
        :param color:ARGB的HEX十六进制颜色值转换后的十进制数值
        :param hoist:在成员列表中单独展示: 0-否, 1-是
        """
        self.name = name
        self.color = color
        self.hoist = hoist


class RoleUpdateResult:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.role_id: str = ""
        self.role: Role = Role()
        if data is not None:
            self.__dict__ = data
